%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Comparison
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;close all;
clc;

%% generate intensity and phase distribution
pixelsize=(2e-3); % 2um pixel size
lambda=633e-6;    % wavelength
k=2*pi/lambda;    % wave_number

eval Generate_Test_Image

figure
subplot(1,2,1)
imshow(I,[]);
title('Intensity');

subplot(1,2,2)
imshow(Phase,[]);
title('Phase');

%% numerical propagation to generate defocused images

U0=sqrt(I).*exp(1i.*Phase);
dz=50e-3;   % defocus distance 10um

Uz  = Numerical_Propagation(U0,dz,pixelsize,lambda,'Angular Spectrum');
Iz=Uz.*conj(Uz);
U_z  = Numerical_Propagation(U0,-dz,pixelsize,lambda,'Angular Spectrum');
I_z=U_z.*conj(U_z);

STD = 0.0005;
I = I+randn(size(I))*STD;
Iz = Iz+randn(size(I))*STD;
I_z = I_z+randn(size(I))*STD;


dIdz=(Iz-I_z)/2/dz; % axial intensity derivative

figure
subplot(1,4,1);
imshow(I_z,[]);
title('I_z');

subplot(1,4,2);
imshow(I,[]);
title('I');

subplot(1,4,3);
imshow(Iz,[]);
title('Iz');

subplot(1,4,4);
imshow(dIdz,[]);
title('dIdz');


%% Solve TIE with iteration function.

RegPara = eps;
IntThr = 0.01;
method = 'DCT';
JudgeFlag = 'delta_dIdzStd';
StdThr = 0.0001;
MaxIterNum = 5;

NumerProp.dz = dz;
NumerProp.lambda = lambda;

I0z = I;

[phi_iter, num_iter] = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr,NumerProp);

% Error Calculation.
phiErr_iter = phi_iter - Phase;
phiErr_iter = phiErr_iter-mean(phiErr_iter(~isnan(phiErr_iter)));


%% crop each image according to the aperture edge (make sure all the boundary signal is included in the cropped region) 
more=0;

Iz=Iz(aperture_up-more:aperture_down+more,aperture_left-more:aperture_right+more);
I_z=I_z(aperture_up-more:aperture_down+more,aperture_left-more:aperture_right+more);
I=I(aperture_up-more:aperture_down+more,aperture_left-more:aperture_right+more);
dIdz=dIdz(aperture_up-more:aperture_down+more,aperture_left-more:aperture_right+more);

Phase=Phase(aperture_up:aperture_down,aperture_left:aperture_right);





phi_iter=phi_iter(aperture_up-more:aperture_down+more,aperture_left-more:aperture_right+more);
phiErr_iter=phiErr_iter(aperture_up-more:aperture_down+more,aperture_left-more:aperture_right+more);
phiErr_iter = phiErr_iter-mean(phiErr_iter(~isnan(phiErr_iter)));


figure;imshow(phi_iter,[]);colormap jet; colorbar;
figure;imshow(Phase,[]);colormap jet; colorbar;

%% Solve TIE
Phi_DCT=DCT_TIE_solver(dIdz,I,pixelsize,k,eps,0.01,more);
Phi_FFT=FFT_TIE_solver(dIdz,I,pixelsize,k,eps,0.01);

figure
subplot(2,2,1);
imshow(Phi_DCT,[]);
title('Phase retrieved by DCT-TIE');

subplot(2,2,2);
imshow(Phi_FFT,[]);
title('Phase retrieved by FFT-TIE');

subplot(2,2,3);
imshow(Phase,[]);
title('True Phase');

subplot(2,2,4);
err=Phi_DCT-Phase;
% imshow(err-mean2(err),[-0.1,0.1]);
imshow(err-mean2(err),[]);
colormap jet;
colorbar
title('DCT-TIE error');



figure;imshow(phiErr_iter,[]);
colormap(WRGBKColorMap);
colorbar;

figure;imshow(err-mean2(err),[]);
colormap(WRGBKColorMap);
colorbar;


