% DemoScript for TIE with arbitrary aperture (Real Experimental Data).

%% Clear everything existing.

clear all;
close all;
clc;

addpath('..\TIE Functions');

%% Setting up system parameters.
I0z_raw = double(imread('I0.bmp'));
Ipz_raw = double(imread('Iz.bmp'));
Inz_raw = double(imread('I_z.bmp'));

% Show Intensities.
I_show = mat2gray([Inz_raw,I0z_raw,Ipz_raw]);
figure('Name','[mat2gray([Inz_raw,I0z_raw,Ipz_raw])]');
imshow(I_show,[]);

%% w Aperture

% crop the image to fit the aperture region
Inz = Inz_raw(117:956,113:1202);
I0z = I0z_raw(117:956,113:1202);
Ipz = Ipz_raw(117:956,113:1202);

pixelsize = 4.65e-6;    % 4.65um pixel size (m)
lambda = 550e-9;        % wavelength (m)
k = 2*pi/lambda;        % wave number
dz = 550e-6;            % defocus distance (m)
reduced_region = 40;    % reduced_region (pixel)

dIdz = (Ipz-Inz)/2/dz; 

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show,[]);

% Check if satisfying Conservation of Energy.
disp(['mean2(dIdz)) = ' num2str(mean2(dIdz))]);
if abs(mean2(dIdz))>1e-7    % Need to check the threshold (1e-7 for now!)
    warning('Conservation of Energy may not be satisfied!');
end

% Show one-step DCT result.................................................
phi_DCT_onestep  = DCT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05,reduced_region);
figure
imshow(phi_DCT_onestep,[]);
% imshow(mod(phi_DCT_onestep,2*pi),[]);
title('phi DCT onestep');
colormap(jet);
colorbar;

% Show one-step FFT result.................................................
phi_FFT_onestep = FFT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05);
figure
imshow(phi_FFT_onestep,[]);
% imshow(mod(phi_FFT_onestep,2*pi),[]);
title('phi FFT onestep');
colormap(jet);
colorbar;

%% w/o aperture

% Show one-step DCT result.
Ipz = Ipz_raw(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);
I0z = I0z_raw(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);
Inz = Inz_raw(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);

% Set the size of CCD.
[Ny, Nx] = size(I0z);
dIdz = (Ipz-Inz)/2/dz; 

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show,[]);

% Show one-step DCT result.
phi_DCT_onestep_wo_aperture = DCT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05,0);
figure
imshow(phi_DCT_onestep_wo_aperture,[]);
% imshow(mod(phi_DCT_onestep_wo_aperture,2*pi),[]);
title('phi DCT onestep, wo aperture');
colormap(jet);
colorbar;

% Show one-step FFT result.
phi_FFT_onestep_wo_aperture = FFT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05);
figure
imshow(phi_FFT_onestep_wo_aperture,[]);
% imshow(mod(phi_FFT_onestep_wo_aperture,2*pi),[]);
title('phi FFT onestep, wo aperture');
colormap(jet);
colorbar;


%% RAW

pause(0.05);

% use raw image for iterative DCT.
Inz = Inz_raw;
I0z = I0z_raw;
Ipz = Ipz_raw;

dIdz = (Ipz-Inz)/2/dz; 
% Check if satisfying Conservation of Energy.
disp(['mean2(dIdz)) = ' num2str(mean2(dIdz))]);
if abs(mean2(dIdz))>1e-7    % Need to check the threshold (1e-7 for now!)
    warning('Conservation of Energy may not be satisfied!');
end

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show,[]);

% Show one-step FFT result.
phi_FFT_onestep_wo_aperture = FFT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05);
figure
imshow(phi_FFT_onestep_wo_aperture,[]);
% imshow(mod(phi_FFT_onestep_wo_aperture,2*pi),[]);
title('phi FFT onestep, wo aperture');
colormap(jet);
colorbar;


%% Solve TIE iter.

RegPara = eps;
IntThr = 0.15;
JudgeFlag = 'delta_dIdzStd';
StdThr = 0.05;
MaxIterNum = 6;

% .........................................................................
% DCT......................................................................
% .........................................................................
method = 'DCT';

% phi_TIE_DCT..............................................................
phi_TIE_DCT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr);
phi_TIE_DCT = phi_TIE_DCT(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);
phiErr_TIE_DCT = RemovePiston(phi_TIE_DCT - phi_DCT_onestep);

figure
imshow(phi_TIE_DCT,[]);
title('phi iterDCT w aperture');
colormap(jet);
colorbar;

figure
imshow(phiErr_TIE_DCT,[]);
title('phiErr TIE DCT w aperture');
colormap(jet);
colorbar;

