% DemoScript for TIE with arbitrary aperture (Real Experimental Data).

%% Clear everything existing.

clear all;
close all;
clc;

addpath('..\TIE Functions');

%% Setting up system parameters.
I0z_raw = double(imread('I0.bmp'));
Ipz_raw = double(imread('Iz.bmp'));
Inz_raw = double(imread('I_z.bmp'));

pixelsize = 4.65e-6;    % 4.65um pixel size (m)
lambda = 550e-9;        % wavelength (m)
k = 2*pi/lambda;        % wave number
dz = 550e-6;            % defocus distance (m)
reduced_region = 40;    % reduced_region (pixel)

figure;imshow(I0z_raw,[0 255]);

figure; hist(I0z_raw(:),128);
h = findobj(gca,'Type','patch');
set(h,'FaceColor','g','EdgeColor','g')
set(gcf,'Position',[100,100,1200,300]);
xlabel('Intensity','FontSize',24)
ylabel('Number of Pixels','FontSize',24);
set(gca,'FontSize',24 ...
    , 'LineWidth', 2 ...
    );


figure;imshow(Ipz_raw,[0 255])
figure;imshow(Inz_raw,[0 255])

BW = I0z_raw>25;
figure;imshow(BW,[])
[B,L] = bwboundaries(BW);
InAperture = L>0&L~=2;
figure;imshow(InAperture,[]);
figure;imshow(I0z_raw,[]);
hold on;
boundary = B{1};
plot(boundary(:,2), boundary(:,1), 'r', 'LineWidth', 5)



%% w Aperture

% crop the image to fit the aperture region
Inz = Inz_raw(117:956,113:1202);
I0z = I0z_raw(117:956,113:1202);
Ipz = Ipz_raw(117:956,113:1202);


dIdz = (Ipz-Inz)/2/dz; 

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show,[]);

% Check if satisfying Conservation of Energy.
disp(['mean2(dIdz)) = ' num2str(mean2(dIdz))]);
if abs(mean2(dIdz))>1e-7    % Need to check the threshold (1e-7 for now!)
    warning('Conservation of Energy may not be satisfied!');
end

% Show one-step DCT result.................................................
phi_DCT_onestep  = DCT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05,reduced_region);
figure
imshow(phi_DCT_onestep,[]);
% imshow(mod(phi_DCT_onestep,2*pi),[]);
title('phi DCT onestep');
colormap(jet);
colorbar;

% Show one-step FFT result.................................................
phi_FFT_onestep = FFT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05);
figure
imshow(phi_FFT_onestep,[]);
% imshow(mod(phi_FFT_onestep,2*pi),[]);
title('phi FFT onestep');
colormap(jet);
colorbar;

%% w/o aperture

% Show one-step DCT result.
Ipz = Ipz_raw(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);
I0z = I0z_raw(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);
Inz = Inz_raw(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);

% Set the size of CCD.
[Ny, Nx] = size(I0z);
dIdz = (Ipz-Inz)/2/dz; 

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show,[]);

% Show one-step DCT result.
phi_DCT_onestep_wo_aperture = DCT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05,0);
figure
imshow(phi_DCT_onestep_wo_aperture,[]);
% imshow(mod(phi_DCT_onestep_wo_aperture,2*pi),[]);
title('phi DCT onestep, wo aperture');
colormap(jet);
colorbar;

% Show one-step FFT result.
phi_FFT_onestep_wo_aperture = FFT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05);
figure
imshow(phi_FFT_onestep_wo_aperture,[]);
% imshow(mod(phi_FFT_onestep_wo_aperture,2*pi),[]);
title('phi FFT onestep, wo aperture');
colormap(jet);
colorbar;

%% RAW

% use raw image for iterative DCT.
Inz = Inz_raw;
I0z = I0z_raw;
Ipz = Ipz_raw;

dIdz = (Ipz-Inz)/2/dz; 
figure;
imshow(dIdz,[])


% Check if satisfying Conservation of Energy.
disp(['mean2(dIdz)) = ' num2str(mean2(dIdz))]);
if abs(mean2(dIdz))>1e-7    % Need to check the threshold (1e-7 for now!)
    warning('Conservation of Energy may not be satisfied!');
end

% Show Intensities.
I_show = [mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)];
figure('Name','[mat2gray([Inz,I0z,Ipz]),mat2gray(dIdz)]');
imshow(I_show,[]);

% Show one-step DCT result.
phi_DCT_onestep_FOV = DCT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05,0);
figure
imshow(phi_DCT_onestep_FOV,[]);
% imshow(mod(phi_DCT_onestep_FOV,2*pi),[]);
title('phi DCT onestep FOV');
colormap(jet);
colorbar;

phi_DCT_onestep_FOV_InAperture = phi_DCT_onestep_FOV;
phi_DCT_onestep_FOV_InAperture(~InAperture)=min(phi_DCT_onestep_FOV_InAperture(InAperture))-range(phi_DCT_onestep_FOV_InAperture(InAperture))/254;
figure;
imshow(phi_DCT_onestep_FOV_InAperture,[]);
% imshow(mod(phi_DCT_onestep_FOV_InAperture,2*pi),[]);
title('phi DCT onestep FOV InAperture');
colormap([[0 0 0];jet(255)]);
colorbar;


% Show one-step FFT result.
phi_FFT_onestep_FOV = FFT_TIE_solver(dIdz,I0z,pixelsize,k,eps,0.05);
figure
imshow(phi_FFT_onestep_FOV,[]);
% imshow(mod(phi_FFT_onestep_FOV,2*pi),[]);
title('phi FFT onestep FOV');
colormap(jet);
colorbar;

phi_FFT_onestep_FOV_InAperture = phi_FFT_onestep_FOV;
phi_FFT_onestep_FOV_InAperture(~InAperture)=min(phi_FFT_onestep_FOV_InAperture(InAperture))-range(phi_FFT_onestep_FOV_InAperture(InAperture))/254;
figure;
imshow(phi_FFT_onestep_FOV_InAperture,[]);
% imshow(mod(phi_FFT_onestep_FOV_InAperture,2*pi),[]);
title('phi FFT onestep FOV InAperture');
colormap([[0 0 0];jet(255)]);
colorbar;


% Solve TIE iter...........................................................

RegPara = eps;
IntThr = 0.15;
JudgeFlag = 'delta_dIdzStd';
StdThr = 0.05;
MaxIterNum = 6;

% .........................................................................
% DCT......................................................................
% .........................................................................
method = 'DCT';

% phi_TIE_DCT..............................................................
phi_TIE_DCT = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method ...
    , MaxIterNum, JudgeFlag, StdThr);

phi_TIE_DCT_cropped = phi_TIE_DCT(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region);

phiErr_TIE_DCT_cropped = RemovePiston(phi_TIE_DCT_cropped - phi_DCT_onestep);

figure
imshow(phi_TIE_DCT,[]);
% imshow(mod(phi_TIE_DCT,2*pi),[]);
title('phi iterDCT FOV');
colormap(jet);
colorbar;

phi_TIE_DCT_InAperture = phi_TIE_DCT;
phi_TIE_DCT_InAperture(~InAperture)=min(phi_TIE_DCT_InAperture(InAperture))-range(phi_TIE_DCT_InAperture(InAperture))/254;
figure;
imshow(phi_TIE_DCT_InAperture,[]);
% imshow(mod(phi_TIE_DCT_InAperture,2*pi),[]);
title('phi iterDCT FOV InAperture');
colormap([[0 0 0];jet(255)]);
colorbar;

%% SHOW TIME
phi_FFT_onestep_FOV_wo_aperture = NaN(size(I0z));
phi_FFT_onestep_FOV_wo_aperture(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region)=phi_FFT_onestep_wo_aperture;
phi_DCT_onestep_FOV_wo_aperture = NaN(size(I0z));
phi_DCT_onestep_FOV_wo_aperture(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region)=phi_DCT_onestep_wo_aperture;

figure;
imshow(dIdz,[]);
hold on;
rectangle('Position', [113+reduced_region,117+reduced_region, 1202-reduced_region-(113+reduced_region), 956-reduced_region-(117+reduced_region)] ...
    , 'LineWidth',5 ...
    , 'EdgeColor',[1 0.5 0.5])
rectangle('Position', [113,117, 1202-(113), 956-(117)] ...
    , 'LineWidth',5 ...
    , 'EdgeColor',[0.5 0.5 1])

figure;
imshow(phiErr_TIE_DCT_cropped,[]);
title('phiErr TIE DCT cropped');
colormap(jet);
colorbar;

Result3= [phi_FFT_onestep_FOV_InAperture,phi_DCT_onestep_FOV_InAperture,phi_TIE_DCT_InAperture];
InAperture3 = [InAperture,InAperture,InAperture];
Result3(~InAperture3) ...
    = min(Result3(InAperture3))-range(Result3(InAperture3))/254;

figure;
imshow(Result3,[]);
% imshow(cos(mod(ResultShow,2*pi)),[]);
title('FOV InAperture');
colormap([[0 0 0];jet(255)]);
colorbar;

figure;
imshow(cos(mod(Result3,2*pi)),[]);


Benchmark = NaN(size(I0z_raw));
Benchmark(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region)=phi_DCT_onestep;
Benchmark(isnan(Benchmark)) ...
    = min(Benchmark(~isnan(Benchmark)))-range(Benchmark(~isnan(Benchmark)))/254;
figure;
imshow(Benchmark,[]);
title('Benchmark InAperture');
colormap([[0 0 0];jet(255)]);
colorbar;

Benchmark = NaN(size(I0z_raw));
Benchmark(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region)=phi_DCT_onestep;

Benchmark3 = [Benchmark,Benchmark,Benchmark];

Error3 = Result3-Benchmark3;
MAX = max(Error3(~isnan(Error3)));
MIN = min(Error3(~isnan(Error3)));
MM = abs(MAX)*(abs(MAX)>=abs(MIN))+abs(MIN)*(abs(MAX)<abs(MIN));
figure;
imshow(Error3,[-MM MM]);
title('FOV InAperture');
colormap(WRGBKColorMap2);
colorbar;


Result2= [phi_FFT_onestep_FOV_wo_aperture,phi_TIE_DCT_InAperture];
MAX = max(Result2(~isnan(Result2)));
MIN = min(Result2(~isnan(Result2)));
MM = abs(MAX)*(abs(MAX)>=abs(MIN))+abs(MIN)*(abs(MAX)<abs(MIN));
InRecon = zeros(size(I0z_raw));
InRecon(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region)=1;

InAperture2 = [InRecon,InAperture];
Result2(~InAperture2) ...
    = min(Result2(InAperture2>0))-range(Result2(InAperture2>0))/254;

figure;
imshow(Result2,[-MM MM]);
% imshow(cos(mod(Result2,2*pi)),[]);
title('FOV InAperture');
colormap([[0 0 0];jet(255)]);
colorbar;

Benchmark = NaN(size(I0z_raw));
Benchmark(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region)=phi_DCT_onestep;
Benchmark(isnan(Benchmark)) ...
    = -MM-2*MM/254;
figure;
imshow(Benchmark,[-MM MM]);
title('Benchmark InAperture');
colormap([[0 0 0];jet(255)]);
colorbar;

Benchmark = NaN(size(I0z_raw));
Benchmark(117+reduced_region:956-reduced_region,113+reduced_region:1202-reduced_region)=phi_DCT_onestep;
Benchmark2 = [Benchmark,Benchmark];

Error2 = Result2-Benchmark2;
MAX = max(Error2(~isnan(Error2)));
MIN = min(Error2(~isnan(Error2)));
MM = abs(MAX)*(abs(MAX)>=abs(MIN))+abs(MIN)*(abs(MAX)<abs(MIN));
figure;
imshow(Error2,[-MM MM]);
title('FOV InAperture');
colormap(WRGBKColorMap2);
colorbar;



Err = Error2(:,1:end/2);
std(Err(~isnan(Err)))

std(phiErr_TIE_DCT_cropped(:))