function [phi_DCT,num,delta_dIdzStd,phi_compStd] = TIEiter(dIdz,I0z,pixelsize,k,RegPara,IntThr,method,MaxIterNum,JudgeFlag,StdThr,NumerProp)

% Check nargin.
if nargin<9
    JudgeFlag = 'delta_dIdzStd';
end

% TIE_solver_meanI
phi_DCT = TIE_solver_meanI(dIdz,I0z,pixelsize,k,RegPara,IntThr,method);

% Iterative Compensation...................................................

% Initialization.
IterNum = 1:MaxIterNum;
delta_dIdzStd = NaN*IterNum;
phi_compStd = NaN*IterNum;

if exist('NumerProp','var')
    [Ny,Nx] = size(I0z);
    dz = NumerProp.dz;
    lambda = NumerProp.lambda;
end
IterNum = 1:MaxIterNum;
for num = IterNum

    
    if exist('NumerProp','var')
        % Expand the Matrices for Numerical Propagration.
        Indy = Ny/2+1:Ny/2*3;
        Indx = Nx/2+1:Nx/2*3;

        I0z_est = zeros(2*Ny,2*Nx);
        I0z_est(Ny/2+1:Ny/2*3,Nx/2+1:Nx/2*3) = I0z;

        phi_DCT_est = zeros(2*Ny,2*Nx);
        phi_DCT_est(Ny/2+1:Ny/2*3,Nx/2+1:Nx/2*3) = phi_DCT;

        % Estimated complex amplitude of optical field in foucs.
        U0z_est = sqrt(I0z_est).*exp(1i.*phi_DCT_est);

        % Numerical propagation for defocused images.
        Unz_est = Numerical_Propagation(U0z_est,-dz,pixelsize,lambda,'Angular Spectrum');
        Upz_est = Numerical_Propagation(U0z_est,+dz,pixelsize,lambda,'Angular Spectrum');

        Inz_est = Unz_est.*conj(Unz_est);
        Ipz_est = Upz_est.*conj(Upz_est);

        % Crop the Matrices.
        Inz_est = Inz_est(Indy,Indx);
        Ipz_est = Ipz_est(Indy,Indx);

        % Estimated dIdz.
        dIdz_est = (Ipz_est-Inz_est)/2/dz;
    else
        % Estimate dIdz with TIE relations.
        dIdz_est = Estimate_dIdz(phi_DCT,I0z,pixelsize,k);
    end
    
    % Update delta_dIdz. 
    delta_dIdz = dIdz - dIdz_est;
    % Check Std.
    delta_dIdzStd(num) = std(delta_dIdz(~isnan(delta_dIdz)));

    % TIE_solver_meanI
    phi_comp = TIE_solver_meanI(delta_dIdz,I0z,pixelsize,k,RegPara,IntThr,method);
    % Check Std.
    phi_compStd(num) = std(phi_comp(~isnan(phi_comp)));
    
    if strcmpi(JudgeFlag,'delta_dIdzStd')
        Judge = delta_dIdzStd(num) < StdThr;
    elseif strcmpi(JudgeFlag,'phi_compStd')
        Judge = phi_compStd(num) < StdThr;
    else
        error('Currently only allow ''delta_dIdzStd'' or ''phi_compStd''.');
    end
    if Judge
        break;
    else
        % update to phi_DCT.
        phi_DCT = phi_DCT + phi_comp;
    end

end

end % End of Function.